%==========================================================================
%
%   Compute Extinction, Scattering, and Absorption Efficienties for 2D
%   Scattering from an Infinite Dielectric Cylinder.
%
%       [Qext, Qsca, Qabs] = Mie_2D_Q(m, x, POL) returns the Mie
%   efficiencies for an infinite dielectric cylinder struck by a plane wave
%   that is normal to the cylinder axis.
%
%   INPUTS:
%       m = Complex index of refraction of the spherical particle, relative
%               to the ambient medium.  That is, m = n1/n, where n1 is the
%               index of the particle and n is the index of the surrounding
%               medium.
%       x = Size parameter (k*a) of the cylinder relative to the ambient
%               medium, where k is the wavenumber in the ambient medium and
%               'a' is the radius of the cylinder.
%       POL = Polarization of incident wave.  If POL == 1, then the
%              incident polarization is assumed to be PARALLEL to the axis
%              of the cylinder.  Otherwise, the polarization is assumed to
%              be PERPENDICULAR to the cylindrical axis.
%
%   OUTPUTS:
%       Qext = Extinction efficiency.
%       Qsca = Scattering efficiency.
%       Qabs = Absorption efficiency.
%
%       This function utilizes the formulation described in "Absorption and
%   Scattering of Light by Small Particles,"  by Bohren and Huffman, 1983.
%   Important information may be found using Equations 8.36 and 8.37, page
%   204.
%
%       REQUIRED FUNCTIONS:
%           Mie_2D_ab.m
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright March 9, 2010

function [Qext, Qsca, Qabs] = Mie_2D_Q(m, x, POL)

%  Transmission amplitudes.
[a b a0 b0] = Mie_2D_ab(m, x);

%   Parallel polarization.  See Equation 8.36 from B&H.
if POL == 1
    
    %  Scattering efficiency.
    Qsca = (2/x)*( abs(b0)^2 + 2*sum( abs(b).^2 ) ); 
  
    %  Extinction efficiency.
    Qext = (2/x)*real( b0 + 2*sum(b) );
    
%   Perpendicular polarization.  See Equation 8.37 from B&H.    
else
    %  Scattering efficiency.
    Qsca = (2/x)*( abs(a0)^2 + 2*sum( abs(a).^2 ) ); 
  
    %  Extinction efficiency.
    Qext = (2/x)*real( a0 + 2*sum(a) );
end

%  Absorption efficiency.
Qabs = Qext - Qsca;


return;


