%==========================================================================
%
%   Compute 2D Scattering Amplitudes for Infinite Cylinder Struck at Normal
%   Incidence.
%
%       [T1, T2] = Mie_2D_T12(m,x,phi) computes the angle-dependent 2D
%   scattering amplitudes.
%
%   INPUTS:
%       m = Complex index of refraction of the spherical particle, relative
%               to the ambient medium.  That is, m = n1/n, where n1 is the
%               index of the particle and n is the index of the surrounding
%               medium.
%       x = Size parameter (k*a) of the cylinder relative to the ambient
%               medium, where k is the wavenumber in the ambient medium and
%               'a' is the radius of the cylinder.
%       phi = Scattering angle in radians.  The direction phi = 0 is the
%               BACKSCATTER direction, with phi = pi corresponding to the
%               FORWARD scattering direction.
%
%   OUTPUTS:
%       T1 = Parallel scattering amplitude.
%       T2 = Perpendicular scattering amplitude.
%
%       This function utilizes the formulation described in "Absorption and
%   Scattering of Light by Small Particles,"  by Bohren and Huffman, 1983.
%   Important information may be found using Equations 8.34, page 202.
%
%       REQUIRED FUNCTIONS:
%           Mie_2D_ab.m
%
%
%       GEOMETRY:
%                                                        \    ^
%                                                         \   | +y
%                               Cylinder                  _\  |
%                                  ===               phi /  \ | 
%            Incident Wave (-x)   /   \                  |   \|
%               -------------->  |     |           +x <--------   
%                                 \   /
%                                  ===
%
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright March 15, 2010

function [T1, T2] = Mie_2D_T12(m, x, phi)

%  Transformed angle variable.
THETA = pi - phi;

%  First compute the Mie coefficients.
[a, b, a0, b0] = Mie_2D_ab(m,x);

%  Extract the number of terms used to compute Mie coefficients.
N_stop = length(a);             %  Truncation length.
n      = [1:N_stop].';          %  Indexing vector.

%  Compute T1 and T2 using equation 8.34 in B & H. 
T1 = b0 + 2*sum( b(n) .* cos(n*THETA) );
T2 = a0 + 2*sum( a(n) .* cos(n*THETA) );

return;


