%==========================================================================
%
%   Compute 2D Cylindrical Scattering Coefficient Vectors
%
%       [a b a0 b0] = Mie_2D_ab(m,x) computes the 2D Mie coefficients for
%   light that is normally incident on an infinite cylinder.
%
%   INPUTS:
%       m = Complex index of refraction of the spherical particle, relative
%               to the ambient medium.  That is, m = n1/n, where n1 is the
%               index of the particle and n is the index of the surrounding
%               medium.
%       x = Size parameter (k*a) of the cylinder relative to the ambient
%               medium, where k is the wavenumber in the ambient medium and
%               'a' is the radius of the cylinder.
%
%   OUTPUTS:
%       a  = Positive a-coefficients (n = 1, 2, 3, ... )
%       b  = Positive b-coefficients (n = 1, 2, 3, ... )
%       a0 = Zero a-term (n = 0).
%       b0 = Zero b-term (n = 0). 
%
%       Keep in mind that this function automatically determines the
%   truncation point for how many relevant coefficients there are, and then
%   automatically returns ALL OF THEM.  This can take a while if m*x is
%   very large, since the function may easily try to compute several
%   hundred values.  So keep in mind that it is probably best to keep the
%   value of x on the order of x < 100.
%
%       This function utilizes the formulation described in "Absorption and
%   Scattering of Light by Small Particles,"  by Bohren and Huffman, 1983.
%   Important information may be found using Equations 8.30 and 8.32, page
%   199.  See also Appendix C, pages 491 - 493 for extra details.
%
%       Finally, note that for negative n-values,
%           a_{-n} = -a_{n},    and     b_{-n} = b_{n}
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright March 9, 2010

function [a b a0 b0] = Mie_2D_ab(m, x)

%  Modified size-parameter.
mx = m*x;

%  Truncation point for computing coefficients.  See Appendix C in B & H,
%  page 491.
N_stop = round( x + 4*x^(1/3) + 2 );

% h-factors for computing the numerical derivatives.
h1 = x*1e-6;
h2 = mx*1e-6;

%  Instantiate scattering coefficients.
a = zeros(N_stop,1);
b = zeros(N_stop,1);

%  Compute scattering coefficients from Equations 8.30 and 8.32 from Bohren
%  and Huffman, page 199.  Note that Matlab is able to compute these
%  directly, thanks to the built-in Bessel functions.  We therefore do not
%  need to bother with the recusion algorithm used by B&H in Appendix C.
for n = 0:N_stop

    %  Pre-compute the derivative terms by using a central-difference
    %  approximation.
    %
    %  J'(x)
    Jp1 = (besselj(n,x + h1)  - besselj(n,x - h1)  ) / (2*h1);     
    
    %  J'(mx)
    Jp2 = (besselj(n,mx + h2) - besselj(n,mx - h2) ) / (2*h2);
    
    %  H^(1)'(x)    
    Hp  = (besselh(n,1,x + h1) - besselh(n,1,x - h1) ) / (2*h1);

    %  Compute numerators.
    Fa = m*besselj(n,mx) * Jp1 - besselj(n,x)*Jp2;
    Fb = besselj(n,mx) * Jp1 - m*Jp2*besselj(n,x);
    
    %  Compute denominators.
    Ga = m*besselj(n,mx) * Hp  - Jp2*besselh(n,1,x);
    Gb = besselj(n,mx) * Hp  - m*Jp2*besselh(n,1,x);
    
    %  Finally compute scattering coefficients.
    a(n+1) = Fa/Ga;
    b(n+1) = Fb/Gb;

end

%  Extract n = 0 terms.
a0 = a(1);
b0 = b(1);

%  Drop the leading term and store the n = 1, 2, 3, ... terms in the
%  vectors.
a = a(2:end);
b = b(2:end);

return;




