%==========================================================================
%
%   Compute Mie Efficiencies for Scattering, Absorption and Extinction.
%
%       [Qext Qsca Qabs] = Mie_3D_Q(n_med, n_sph, x) computes the
%   extinction, scattering, and absorption efficiencies (Qext, Qsca, and
%   Qabs) for a spherical particle embedded in some ambient medium.  Values
%   are computed using the formulation described in "Absorption and
%   Scattering  of Light by Small Particles," by Bohren and Huffman, 1983.
%   See equations 4.61, and 4.62 on page 103. 
%
%   INPUTS:
%       n_med = Complex index of refraction (n' + j*n") of the ambient
%               medium.
%       n_sph = Complex index of refraction (n' + j*n") of the spherical
%               particle.
%       x     = Size parameter of the sphere, where x = k*r. 'k' is the
%               wavenumber in ambient medium and 'r' is radius of the 
%               sphere.  Note that k (and therefore x) can be complex for
%               lossy media.  For example, if k0 is the wavenumber in
%               free-space, then k = k0*n_med.
%
%   OUTPUTS:
%       Qext = Extinction efficiency.
%       Qsca = Scattering efficiency.
%       Qabs = Absorption efficiency.
%
%	REQUIRED FUNCTIONS:
%           Mie_3D_ab.m
%
%       Note that if n_med (and therefore x) is a complex number, then the
%   formulation is slightly different from the standard Bohren and Huffman
%   equations.  In this special case, the formulation is taken from the
%   following reference:
%
%       I. W. Sudiarta and P. Chylek, "Mie-scattering formalism for
%   spherical particles embedded in an absorbing medium," J. Opt. Soc. Am.
%   A, Vol 18, No. 6, June 2001.
%
%   Bear in mind that I have simplified Sudiarta's math for the sake of
%   writing this code, which means the expressions are formulated
%   differently.  Nevertheless, the results are equivalent and may be
%   readily validated against his paper.
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright October 18, 2010

function [Qext, Qsca, Qabs] = Mie_3D_Q(n_med, n_sph, x)

%  Check for errors in input.
if length(n_med) > 1
    error('Error:  n_med must be a scalar.');
end
if length(n_sph) > 1
    error('Error:  n_sph must be a scalar.');
end
if length(x) > 1
    error('Error:  x must be a scalar.');
end
    

%  Prevents any singularities that occur at a = 0.
if x == 0
    Qext = 0;
    Qsca = 0;
    Qabs = 0;
    return;
end

%  Compute relative index of refraction for particle with respect to host
%  medium. 
m = n_sph / n_med;

%  Compute the Mie a-b coefficients.
[a, b] = Mie_3D_ab(m,x);

%  Extract vector size.
N = length(a);

%  If x is a real number, then use standard Bohren and Huffman equations.
if imag(x) == 0
    
    %  Summation indices.
    n = [1:N];

    %  Leading coefficients in summations.
    cn = 2*n + 1;
    cn = cn.';

    %  Coefficient magnitudes.
    A = abs(a).^2;
    B = abs(b).^2;

    %  See equations 4.61 and 4.62.  By dividing by x^2, we are able to
    %  skip over the cross sections and directly compute the Mie
    %  efficiencies.
    q    = 2/x^2;                      % Leading coefficient.
    Qsca = q*sum( cn.*(A + B) );
    Qext = q*sum( cn.*real(a + b) );

    %  Store absorption efficiency.
    Qabs = Qext - Qsca;
    
else

    %  If x is complex, use Sudiarta's modified formulations.  This will
    %  take significantly longer because we now have to re-compute the
    %  Riccati-Bessel/Hankel functions.

    %  Instantiate summation parameters.
    Yext = 0;
    Yabs = 0;
    Ysca = 0;

    for n = 1:N

        cn = 2*n + 1;

        %  Evaluate Riccati-Bessel function and its derivative at x.
        Psi  = riccati_bessel(n,x);
        dPsi = riccati_bessel_dx(n,x);

        %  Evaluate Riccati-Hankel function and its derivative at x.
        Xi  = riccati_hankel(n,x);
        dXi = riccati_hankel_dx(n,x);

        %  Prepare summation terms.  See Sudiarta, 2001.
        A = j*conj(Psi)*dPsi           - j*Psi*conj(dPsi);
        B = j*b(n)*conj(dPsi)*Xi       + j*conj(b(n))*Psi*conj(dXi);
        C = j*abs(a(n))^2*dXi*conj(Xi) - j*abs(b(n))^2*Xi*conj(dXi);
        D = -j*a(n)*conj(Psi)*dXi      - j*conj(a(n))*dPsi*conj(Xi);

        %  Add terms to the summation.
        Yext = Yext + cn*( A + B + D );
        Yabs = Yabs + cn*( A + B + C + D );
        Ysca = Ysca + cn*( - C );

    end

    %  Prepare final coefficients.
    mr = real(n_med);
    xI = imag(x);
    
    %  This next term behaves very poorly for small values of xI, so
    %  replace it with the limiting value for tiny inputs.
    if xI < 1e-6
        Z = 0.5;            %  Limiting value for small xI.
    else
        %  This is the true expression for Z.
        Z = exp(2*xI) / (2*xI) + (1 - exp(2*xI) )/(4*xI^2);
    end
    
    %  One last parameter.
    A = (1/abs(x)^2)*(1/mr)/Z;

    %  Finally, compute the Mie efficiencies.  See Sudiarta, 2001.  
    Qext = A*real( conj(n_med)*Yext );
    Qabs = A*real( conj(n_med)*Yabs );
    Qsca = A*real( conj(n_med)*Ysca );
    
end


%==========================================================================
%  Riccati-Bessel function.
%       n = Function order
%       x = argument
%==========================================================================
function S = riccati_bessel(n,x)

%  Riccati-Bessel function definition.
S = sqrt(x*pi/2) * besselj(n+1/2,x);

return;

%==========================================================================
%  Riccati-Bessel function derivative.
%       n = Function order
%       x = argument
%==========================================================================
function dPsi = riccati_bessel_dx(n,x)

%  Riccati-Bessel function definition.
a = sqrt(x*pi/2);
b = sqrt(pi/2./x)*(n);

dPsi = a .* besselj(n-1/2,x) - b .* besselj(n+1/2,x);

return;


%==========================================================================
%   Riccati-Hankel function.
%       n = Function order
%       x = argument
%==========================================================================
function E = riccati_hankel(n,x)

%  Riccati-Bessel function definition.
E = x.*sqrt(pi./(2*x)).*besselh(n+1/2,x);

return;

%==========================================================================
%   Derivative of Riccati-Hankel function.
%       n = Function order
%       x = argument
%==========================================================================
function dXi = riccati_hankel_dx(n,x)

%  Leading coefficient terms.
a = sqrt(x*pi/2);
b = sqrt(pi/2./x)*(n);

%  Derivative.
dXi = a .* besselh(n-1/2,x) - b .* besselh(n+1/2,x);

return;

