%==========================================================================
%
%   Compute Angle-Dependent 3D Scattering Amplitudes, S1 and S2.
%
%       [S1, S2] = Mie_3D_S12(m,x,theta) computes the angle-dependent
%   scattering amplitudes from Mie theory.  Values are computed using
%   the formulation described in "Absorption and Scattering of Light by
%   Small Particles,"  by Bohren and Huffman, 1983. See equation 4.74 on
%   page 112.  These functions are necessary for calculating the Stokes
%   parameters of scattered light.
%
%   INPUTS:
%       m = Complex index of refraction of the spherical particle, relative
%               to the ambient medium.  That is, m = n_sphere / n_medium.  
%       x = Size parameter (k_0*a) of the sphere relative to the ambient
%               medium, where k_0 is the wavenumber in ambient medium and
%               'a' is the radius of the sphere. 
%       theta = Scattering angle (in radians).
%
%   OUTPUTS:
%       S1 = Scattering amplitude for perpendicular polarization.
%       S2 = Scattering amplitude for parallel polarization.
%
%   Note that polarization is referenced against the *scattering plane.*
%
%   The incident plane wave satisfies:
%
%       E_||i = cos(phi)*E_xi + sin(phi)*E_yi   (parallel component)
%       E_|_i = sin(phi)*E_xi - cos(phi)*E_yi   (perpendicular component)
%
%   In the scattered fields, the parallel and perpendicular unit vectors
%   are along the THETA and PHI coordinates:
%
%       PARALLEL COMPONENT      = theta
%       PERPENDICULAR COMPONENT = - phi
%
%   GEOMETRY:
%                             +z 
%                               ^     /
%                               | TH /
%                               |---/
%                               |  /
%                               | /
%                               |/
%                               ----------> +y
%
%       Spherical particle is centered at the origin.  The incident wave is
%   assumed to be propagating in the +z direction.  The angle THETA (TH) is
%   referenced against the +z-axis.
%
%       REQUIRED FUNCTIONS:
%           Mie_3D_ab.m
%           Mie_pi_tau.m
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright March 15, 2010

function [S1, S2] = Mie_3D_S12(m, x, theta)

%  First compute the Mie coefficients.
[a, b] = Mie_3D_ab(m,x);

%  Extract the number of terms used to compute Mie coefficients.
N_stop = length(a);
n      = [1:N_stop].';        %  Indexing vector.

%  Compute pi/tau functions.
[pi, tau] = Mie_pi_tau(theta,N_stop);

%  Leading coefficients in summation.
cn = (2*n + 1)./(n .* (n + 1) );

%  Compute S1 and S2 using equation 4.74 in B & H. 
S1 = sum( cn.*(a.*pi + b.*tau ) );
S2 = sum( cn.*(a.*tau + b.*pi ) );

return;


