%==========================================================================
%
%   Compute Mie Coefficient Vectors, a_n and b_n.
%
%       [a, b] = Mie_3D_ab(m,x) computes two vectors of Mie coefficients.
%   Values are computed using the formulation described in "Absorption and
%   Scattering of Light by Small Particles,"  by Bohren and Huffman, 1983.
%   See equations 4.56 and 4.57 on page 101.
%
%   INPUTS:
%       m = Relative index of refraction for particle with respect to host
%           medium.  That is, m = n_sphere / n_medium.  
%       x = Size parameter (k*r) of the sphere relative to the ambient
%           medium, where k is the wavenumber in ambient medium and
%         	'r' is the radius of the sphere.  Note that x can be complex.
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright October 18, 2010

function [a, b] = Mie_3D_ab(m,x)

%  Compute product between index and size parameter.
mx = m*x;

%  Truncation point for computing coefficients.  See Appendix A in B & H,
%  page 477.
N_STOP = round( abs(x + 4*x^(1/3) + 2 ) );

%  Termination condition for coefficients.  When the latest coefficient in
%  the series is some tiny fraction of the first coeffcient, that is, when
%  a(n) < err*a(1) and b(n) < err*b(1), then the series is assumed to have
%  converged and the calculations terminate.
err = 1e-6;

%  Instantiate scattering coefficients.
a = zeros(N_STOP,1);
b = zeros(N_STOP,1);

%  Compute scattering coefficients from Equations 8.30 and 8.32 from Bohren
%  and Huffman, page 199.  Note that Matlab is able to compute these
%  directly, thanks to the built-in Bessel functions.  We therefore do not
%  need to bother with the recusion algorithm used by B&H in Appendix C.
for n = 1:N_STOP

    %  Evaluate Riccati-Bessel function and its derivative at m*x.
    S1  = riccati_bessel(n,mx);
    dS1 = riccati_bessel_dx(n,mx);

    %  Repeat above at x.
    S2  = riccati_bessel(n,x);
    dS2 = riccati_bessel_dx(n,x);
    
    %  Evaluate Riccati-Hankel function and its derivative at m*x.
    E1  = riccati_hankel(n,x);
    dE1 = riccati_hankel_dx(n,x);

    %  Compute numerators.
    Fa = m*S1*dS2 -   S2*dS1;
    Fb =   S1*dS2 - m*S2*dS1;

    %  Compute denominators.
    Ga = m*S1*dE1 -   E1*dS1;
    Gb =   S1*dE1 - m*E1*dS1;

    %  Finally compute scattering coefficients.
    a(n) = Fa/Ga;
    b(n) = Fb/Gb;

    %  Check for early convergence and terminate.
    diff1 = abs(a(n)) / abs(a(1));
    diff2 = abs(b(n)) / abs(b(1));
    if ( diff1 < err ) && ( diff2 < err )
        a = a(1:n);
        b = b(1:n);
        break;
    end

end

return;

%==========================================================================
%  Riccati-Bessel function.
%       n = Function order
%       x = argument
%==========================================================================
function S = riccati_bessel(n,x)

%  Riccati-Bessel function definition.
S = sqrt(x*pi/2) * besselj(n+1/2,x);

return;

%==========================================================================
%  Riccati-Bessel function derivative.
%       n = Function order
%       x = argument
%==========================================================================
function dS = riccati_bessel_dx(n,x)

%  Coefficients.
a = sqrt(x*pi/2);
b = sqrt(pi/2./x)*(n);

%  This is how you compute the derivative.
dS = a .* besselj(n-1/2,x) - b .* besselj(n+1/2,x);

return;


%==========================================================================
%   Riccati-Hankel function.
%       n = Function order
%       x = argument
%==========================================================================
function E = riccati_hankel(n,x)

E = x.*sqrt(pi./(2*x)).*besselh(n+1/2,x);

return;

%==========================================================================
%   Derivative of Riccati-Hankel function.
%       n = Function order
%       x = argument
%==========================================================================
function dE = riccati_hankel_dx(n,x)

%  Leading coefficient terms.
a = sqrt(x*pi/2);
b = sqrt(pi/2./x)*(n);

%  This is how you compute the derivative.
dE = a .* besselh(n-1/2,x) - b .* besselh(n+1/2,x);

return;


