%==========================================================================
%
%   Compute Mie Coefficient Vectors, a_n and b_n, using the recursive
%   formulation from Bohren and Huffman (appendix A, page 477).
%
%       [a, b] = Mie_3D_ab_BH(m,x) computes two vectors of Mie
%   coefficients.  Values are computed using the formulation described in
%   "Absorption and Scattering of Light by Small Particles,"  by Bohren and
%   Huffman, 1983.  See equation 4.88 on page 127, and appendix A, page
%   477.
%
%   INPUTS:
%       m = Complex index of refraction of the spherical particle, relative
%               to the ambient medium.  That is, m = n_sphere / n_medium. 
%       x = Size parameter (k_0*a) of the sphere relative to the ambient
%               medium, where k_0 is the wavenumber in ambient medium and
%               'a' is the radius of the sphere. 
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright May 17, 2010

function [a, b] = Mie_3D_ab_BH(m,x)

%  Extra padding for calculating D through reverse recursion.  B&H uses
%  roughly this value on page 478.
N_pad = 16;

%  Input argument for the logarithmic derivative function.
rho = m*x;

%  Truncation point for computing coefficients.  See Appendix A in B & H,
%  page 477.
N_stop = round(abs(x + 4*x^(1/3) + 2 ));

%  Number of recursive terms to use for D.  Convergence requires that N_mx
%  be "sufficiently greater than N_stop and |mx|" (B&H, page 478).  
N_mx = round(max(N_stop,abs(rho)) + N_pad);

%  Instantiate the logarithmic derivative function.  Recurrence begins with
%  D(N_mx) = 0 + j*0.  
D = zeros(N_mx,1);

% Computate D using a backwards a recurrance relation.
for n = N_mx:-1:2

    %  See equation 4.89 in B & H.
    D(n-1) = n/rho - 1/( D(n) + n/rho );
    
end

% Subscript indices for Bessel functions.
n  = [1:N_stop].'; 	%  Integer steps.
nu = n + 0.5;       %  Half-integer steps.

%  The extra terms in D are simply used to help us with convergence, and
%  are therefore no longer needed at this point.
D = D(n);

%  Riccati-Bessel functions.
c  =  sqrt(0.5*pi*x);           %  Leading coefficient.
Sn =  c .* besselj(nu,x);       %  First kind.
Cn = -c .* bessely(nu,x);       %  Second kind.
En = Sn - j*Cn;                 %  Third kind.

%  The {n-1} terms begin at zero with S_0 = sin(x), C_0 = cos(x).  
Sx = [ sin(x); Sn(1:N_stop-1) ];        %  S_{n-1}
Cx = [ cos(x); Cn(1:N_stop-1) ];        %  C_{n-1}
Ex = Sx - j*Cx;                         %  E_{n-1}

%  Numerator/denominator elements for a_n and b_n.  See equation 4.88
%  from Bohren and Huffman.
Fa = D/m + n/x;
Fb = m*D + n/x;

%  Compute Mie coefficients.
a = ( Fa.*Sn - Sx ) ./ ( Fa.*En - Ex ) ;
b = ( Fb.*Sn - Sx ) ./ ( Fb.*En - Ex ) ;



