%==========================================================================
%
%   Compute Mie Efficiencies for Coated Spheres.
%
%       [Qext Qsca Qabs] = Mie_3D_coated_Q(n, n1, n2, x, y) computes the
%   extinction, scattering, and absorption efficiencies (Qext, Qsca, and
%   Qabs) for a COATED spherical dielectric particle embedded in some
%   arbitrary (lossless) dielectric medium.  Values are computed using the
%   formulation described in "Absorption and Scattering  of Light by Small
%   Particles," by Bohren and Huffman, 1983.  See equations 4.61, and 4.62
%   on page 103.  For extension to coated spheres, see equation 8.2 on page
%   183. 
%
%   INPUTS:
%       n0 = REAL-VALUED index of refraction of the ambient medium.
%       n1 = Complex index of refraction (n' + j*n") of the CORE of the
%               spherical particle.
%       n2 = Complex index of refraction (n' + j*n") of the SHELL around
%               core of the spherical particle.
%       x  = Size parameter of the core, where x = k*r. 'k' is the
%               wavenumber in ambient medium and 'r' is radius of the
%               spherical core.  Note that k (and therefore x) can be
%               complex for lossy media, but has not been formulated yet
%               and will give bogus results.
%       y  = Size parameter of the shell, where y = k*R. 'k' is the
%               wavenumber in ambient medium and 'R' is outer radius of the
%               spherical shell.  Again, note that k (and therefore y) can
%               be complex for lossy media, but has not been formulated yet
%               and will give bogus results.
%
%   OUTPUTS:
%       Qext = Extinction efficiency.
%       Qsca = Scattering efficiency.
%       Qabs = Absorption efficiency.
%
%	REQUIRED FUNCTIONS:
%           Mie_3D_coated_ab.m
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright OCtober 18, 2010

function [Qext, Qsca, Qabs] = Mie_3D_coated_Q(n0, n1, n2, x, y)

%  Prevents any singularities that occur at a = 0.
if x == 0
    Qext = 0;
    Qsca = 0;
    Qabs = 0;
    return;
end

%  Compute relative index of refraction for particle with respect to host
%  medium. 
m1 = n1 / n0;
m2 = n2 / n0;

%  Compute the Mie a-b coefficients.
[a, b] = Mie_3D_coated_ab(m1,m2,x,y);

%  Extract vector size.
N = length(a);

%  If x is a real number, then use standard Bohren and Huffman equations.
if imag(x) == 0
    
    %  Summation indices.
    n = [1:N];

    %  Leading coefficients in summations.
    cn = 2*n + 1;
    cn = cn.';

    %  Coefficient magnitudes.
    A = abs(a).^2;
    B = abs(b).^2;

    %  See equations 4.61 and 4.62.  By dividing by x^2, we are able to
    %  skip over the cross sections and directly compute the Mie
    %  efficiencies.
    q    = 2/y^2;                      % Leading coefficient.
    Qsca = q*sum( cn.*(A + B) );
    Qext = q*sum( cn.*real(a + b) );

    %  Store absorption efficiency.
    Qabs = Qext - Qsca;
    
else
    error('Ambient medium cannot be lossy.');
end







