%==========================================================================
%
%   Compute Mie Coefficient Vectors, a_n and b_n for COATED sphere.
%
%       [a, b] = Mie_3D_coated_ab(m1,m2,x,y) computes two vectors of Mie
%   coefficients for a COATED particle in some ambient medium.  Values are
%   computed using the formulation described in "Absorption and Scattering
%   of Light by Small Particles,"  by Bohren and Huffman, 1983. See
%   equation 8.2 on page 183.
%
%   INPUTS:
%      m1 = Refractive index of the CORE relative to the surrounding
%           medium; ie, m1 = n_core / n_medium .
%      m2 = Refractive index of the SHELL relative to the surrounding
%           medium; ie, m1 = n_core / n_medium .
%           medium.  That is, m = n_sphere / n_medium.  
%       x = Size parameter (k*a) of the CORE relative to the ambient
%           medium, where k is the wavenumber in ambient medium and
%         	'a' is the radius of the core.
%       y = Size parameter (k*b) of the SHELL relative to the ambient
%           medium, where k is the wavenumber in ambient medium and
%         	'b' is the radius of the shell.  Note that b > a.
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright October 18, 2010

function [a, b] = Mie_3D_coated_ab(m1,m2,x,y)

%  Precompute Bessel-function arguments.
m1x = m1*x;
m2y = m2*y;
m2x = m2*x;

%  Truncation point for computing coefficients.  See Appendix A in B & H,
%  page 477.
N_STOP = round( abs(y + 4*y^(1/3) + 2 ) );

%  Termination condition for coefficients.  When the latest coefficient in
%  the series is some tiny fraction of the first coeffcient, that is, when
%  a(n) < err*a(1) and b(n) < err*b(1), then the series is assumed to have
%  converged and the calculations terminate.
err = 1e-6;

%  Instantiate scattering coefficients.
a = zeros(N_STOP,1);
b = zeros(N_STOP,1);

%  Compute scattering coefficients from Equations 8.30 and 8.32 from Bohren
%  and Huffman, page 199.  Note that Matlab is able to compute these
%  directly, thanks to the built-in Bessel functions.  We therefore do not
%  need to bother with the recusion algorithm used by B&H in Appendix C.
for n = 1:N_STOP

    %  Evaluate Riccati-Bessel function at required arguments.
    Psi_y   = riccati_bessel_Psi(n,y);
    Psi_m1x = riccati_bessel_Psi(n,m1x);
    Psi_m2x = riccati_bessel_Psi(n,m2x);
    Psi_m2y = riccati_bessel_Psi(n,m2y);
    
    %  Evaluate Riccati-Bessel functions of second kind.
    Chi_m2x = riccati_bessel_Chi(n,m2x);
    Chi_m2y = riccati_bessel_Chi(n,m2y);
    
    %  Evaluate Riccati-Hankel functions.
    Xi_y = riccati_hankel(n,y);
    
    %  Now evaluate all the derivatives.
    dPsi_y   = riccati_bessel_Psi_dx(n,y);                  
    dPsi_m1x = riccati_bessel_Psi_dx(n,m1x);
    dPsi_m2x = riccati_bessel_Psi_dx(n,m2x);
    dPsi_m2y = riccati_bessel_Psi_dx(n,m2y);
    dChi_m2x = riccati_bessel_Chi_dx(n,m2x);
    dChi_m2y = riccati_bessel_Chi_dx(n,m2y);
    dXi_y    = riccati_hankel_dx(n,y);

    %  Start computing sub-coefficients.  Begin with numerators.  See
    %  equation 8.2 in B&H.
    FA = m2*Psi_m2x*dPsi_m1x - m1*dPsi_m2x*Psi_m1x;
    FB = m2*Psi_m1x*dPsi_m2x - m1*Psi_m2x*dPsi_m1x;

    %  Compute denominators.
    GA = m2*Chi_m2x*dPsi_m1x - m1*dChi_m2x*Psi_m1x;
    GB = m2*dChi_m2x*Psi_m1x - m1*Chi_m2x*dPsi_m1x;

    %  Compute sub-coefficients.
    An = FA / GA;
    Bn = FB / GB;

    %  Now start computing the final coefficients, beginning with
    %  numerators.
    Fa =  Psi_y    *( dPsi_m2y - An*dChi_m2y ) ...
        - m2*dPsi_y*( Psi_m2y  - An*Chi_m2y  );
    Fb =  m2*Psi_y *( dPsi_m2y - Bn*dChi_m2y ) ...
        -    dPsi_y*( Psi_m2y  - Bn*Chi_m2y  );

    %  Denominators.
    Ga =  Xi_y    *( dPsi_m2y - An*dChi_m2y ) ...
        - m2*dXi_y*( Psi_m2y  - An*Chi_m2y  );
    Gb =  m2*Xi_y *( dPsi_m2y - Bn*dChi_m2y ) ...
        -    dXi_y*( Psi_m2y  - Bn*Chi_m2y  );

    %  Finally, compute scattering coefficients.
    a(n) = Fa/Ga;
    b(n) = Fb/Gb;

    %  Check for early convergence and terminate.
    diff1 = abs(a(n)) / abs(a(1));
    diff2 = abs(b(n)) / abs(b(1));
    if ( diff1 < err ) && ( diff2 < err )
        a = a(1:n);
        b = b(1:n);
        break;
    end

end

return;

%==========================================================================
%  Riccati-Bessel function of the first kind.
%       n = Function order
%       x = argument
%==========================================================================
function Psi = riccati_bessel_Psi(n,x)

%  Riccati-Bessel function definition.
Psi = sqrt(x*pi/2) * besselj(n+1/2,x);

return;

%==========================================================================
%  Riccati-Bessel function derivative.
%       n = Function order
%       x = argument
%==========================================================================
function dPsi = riccati_bessel_Psi_dx(n,x)

%  Coefficients.
a = sqrt(x*pi/2);
b = sqrt(pi/2./x)*(n);

%  This is how you compute the derivative.
dPsi = a .* besselj(n-1/2,x) - b .* besselj(n+1/2,x);

return;

%==========================================================================
%  Riccati-Bessel function of second kind.
%       n = Function order
%       x = argument
%==========================================================================
function Chi = riccati_bessel_Chi(n,x)

%  Riccati-Bessel function definition.
Chi = -sqrt(x*pi/2) * bessely(n+1/2,x);

return;


%==========================================================================
%  Riccati-Bessel function of second kind derivative.
%       n = Function order
%       x = argument
%==========================================================================
function dChi = riccati_bessel_Chi_dx(n,x)


%  Leading coefficient terms.
a = -sqrt(x*pi/2);
b = -sqrt(pi/2./x)*(n);

%  This is how you compute the derivative.
dChi = a .* bessely(n-1/2,x) - b .* bessely(n+1/2,x);

return;

%==========================================================================
%   Riccati-Hankel function.
%       n = Function order
%       x = argument
%==========================================================================
function Xi = riccati_hankel(n,x)

%  Compute spherical Bessel function.
f = sqrt(pi/(2*x));
h_n = f * besselj(n+1/2,x) + (1i*f) * bessely(n+1/2,x);

%  Riccati-Hankel definition.
Xi = x*h_n;

return;

%==========================================================================
%   Derivative of Riccati-Hankel function.
%       n = Function order
%       x = argument
%==========================================================================
function dXi = riccati_hankel_dx(n,x)

%  Leading coefficient terms.
a = sqrt(x*pi/2);
b = sqrt(pi/2./x)*(n);

%  This is how you compute the derivative.
dXi = a .* besselh(n-1/2,x) - b .* besselh(n+1/2,x);

return;


