%==========================================================================
%
%   Compute Angle-Dependent Mie Functions, pi_n and tau_n, from Bohren and
%   Huffman, pp 94-95.
%
%       [pi, tau] = Mie_pi_tau(theta,N_stop) computes two vectors of
%   angle-dependent Mie functions.  Values are computed using the
%   formulation described in "Absorption and Scattering of Light by Small
%   Particles,"  by Bohren and Huffman, 1983. See equation 4.47 on pages
%   94-95.   
%
%   INPUTS:
%       theta  = Angle (in radians) away from z-axis (ie, spherical
%                   coordinate component).
%       N_stop = Total number of recursion terms to calculate.
%
%   OUTPUTS:
%       pi  = Pi-function.
%       tau = Tau-function.
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright March 15, 2010


function [pi, tau] = Mie_pi_tau(theta,N_stop)

%  Cosine parameter.
mu = cos(theta);

%  Instantiate pi and tau functions.
pi  = zeros(N_stop,1);
tau = zeros(N_stop,1);

%  Recursion initiators.  See B & H, page 95.  We need the n = [1, 2] terms
%  in order to begin the recusive computations.
pi(1)  = 1;                         %  pi_1
pi(2)  = 3*mu;                      %  pi_2
tau(1) = mu;                        %  tau_1
tau(2) = 2*mu*pi(2) - 3*pi(1);      %  tau_2

%  Begin recursion calculations.
for n = 3:N_stop

    % See quation 4.47 in B&H, page 95.
    p1 = ( mu * pi(n-1) ) * (2*n - 1)/(n - 1);
    p2 = pi(n-2) * n/(n - 1);
    pi(n) = p1 - p2;                %  Pi-function.
    
    t1 = n * mu * pi(n);
    t2 = (n + 1) * pi(n-1);
    tau(n) = t1 - t2;               %  Tau-function.
    
end

return;


